[Date Prev][Date Next][Thread Prev][Thread Next][Date Index][Thread Index]

[minion-cvs] Tag refactoring; email robustness.



Update of /home/minion/cvsroot/src/minion/lib/mixminion
In directory moria.seul.org:/tmp/cvs-serv25421/lib/mixminion

Modified Files:
	BuildMessage.py Modules.py Packet.py PacketHandler.py 
	ServerMain.py benchmark.py test.py testSupport.py 
Log Message:
Tag refactoring; email robustness.

BuildMessage, Modules, Packet, PacketHandler, ServerMain, test, testSupport,
   benchmark:
    - Separate 'tag' and 'exitInfo' as much as possible
    - Require all exit messages to have a tag.
    - Insist that exit types be 0x0000 or >= 0x0100.

Packet, Modules, test:
    - Catch invalid email addresses.


Index: BuildMessage.py
===================================================================
RCS file: /home/minion/cvsroot/src/minion/lib/mixminion/BuildMessage.py,v
retrieving revision 1.16
retrieving revision 1.17
diff -u -d -r1.16 -r1.17
--- BuildMessage.py	16 Oct 2002 23:12:11 -0000	1.16
+++ BuildMessage.py	22 Nov 2002 21:12:05 -0000	1.17
@@ -126,6 +126,8 @@
 def buildStatelessReplyBlock(path, exitType, exitInfo, userKey,
 			     expiryTime=0, secretRNG=None):
     """XXXX DOC IS NOW WRONG HERE
+       (exitInfo doesn't include tag)
+
        Construct a 'stateless' reply block that does not require the
        reply-message recipient to remember a list of secrets.
        Instead, all secrets are generated from an AES counter-mode
@@ -144,6 +146,7 @@
                   userKey: an AES key to encrypt the seed, or None.
                   email: If true, delivers via SMTP; else delivers via MBOX
        """
+
     #XXXX Out of sync with the spec.
     if secretRNG is None: secretRNG = Crypto.AESCounterPRNG()
 
@@ -261,6 +264,11 @@
     if isinstance(path2, ReplyBlock):
         reply = path2
         path2 = None
+    else:
+	if len(exitInfo) < TAG_LEN:
+	    raise MixError("Implausibly short exit info: %r"%exitInfo)
+	if exitType < Modules.MIN_EXIT_TYPE and exitType != Modules.DROP_TYPE:
+	    raise MixError("Invalid exit type: %4x"%exitType)
 
     ### SETUP CODE: let's handle all the variant cases.
 

Index: Modules.py
===================================================================
RCS file: /home/minion/cvsroot/src/minion/lib/mixminion/Modules.py,v
retrieving revision 1.16
retrieving revision 1.17
diff -u -d -r1.16 -r1.17
--- Modules.py	21 Nov 2002 18:26:12 -0000	1.16
+++ Modules.py	22 Nov 2002 21:12:05 -0000	1.17
@@ -91,7 +91,7 @@
 	   batch messages intended for the same destination."""
         return SimpleModuleDeliveryQueue(self, queueDir)
 
-    def processMessage(self, message, exitType, exitInfo):
+    def processMessage(self, message, tag, exitType, exitInfo):
 	"""Given a message with a given exitType and exitInfo, try to deliver
            it.  Return one of:
             DELIVER_OK (if the message was successfully delivered),
@@ -108,9 +108,9 @@
     def __init__(self, module):
 	self.module = module
 
-    def queueMessage(self, (exitType, exitInfo), message):
+    def queueMessage(self, (exitType, address, tag), message):
 	try:
-	    res = self.module.processMessage(message, exitType, exitInfo)
+	    res = self.module.processMessage(message, exitType, tag, address)
 	    if res == DELIVER_OK:
 		return
 	    elif res == DELIVER_FAIL_RETRY:
@@ -135,8 +135,8 @@
     def deliverMessages(self, msgList):
 	for handle, addr, message, n_retries in msgList:	
 	    try:
-		exitType, exitInfo = addr
-		result = self.module.processMessage(message,exitType,exitInfo)
+		exitType, address, tag = addr
+		result = self.module.processMessage(message,tag,exitType,address)
 		if result == DELIVER_OK:
 		    self.deliverySucceeded(handle)
 		elif result == DELIVER_FAIL_RETRY:
@@ -277,14 +277,14 @@
 	if self.queues.has_key(module.getName()):
 	    del self.queues[module.getName()]
 
-    def queueMessage(self, message, exitType, exitInfo):
+    def queueMessage(self, message, tag, exitType, address):
         mod = self.typeToModule.get(exitType, None)
         if mod is None:
             getLog().error("Unable to handle message with unknown type %s",
                            exitType)
 	    return
 	queue = self.queues[mod.getName()]
-	queue.queueMessage((exitType,exitInfo), message)
+	queue.queueMessage((exitType, address, tag), message)
 
     def sendReadyMessages(self):
 	for name, queue in self.queues.items():
@@ -308,7 +308,7 @@
         return [ DROP_TYPE ]
     def createDeliveryQueue(self, directory):
 	return ImmediateDeliveryQueue(self)
-    def processMessage(self, message, exitType, exitInfo):
+    def processMessage(self, message, tag, exitType, exitInfo):
         getLog().debug("Dropping padding message")
         return DELIVER_OK
 
@@ -386,16 +386,16 @@
     def getExitTypes(self):
         return [ MBOX_TYPE ]
 
-    def processMessage(self, message, exitType, exitInfo):
+    def processMessage(self, message, tag, exitType, address):
         assert exitType == MBOX_TYPE
         getLog().trace("Received MBOX message")
-        info = mixminion.Packet.parseMBOXInfo(exitInfo)
+        info = mixminion.Packet.parseMBOXInfo(address)
 	try:
 	    address = self.addresses[info.user]
 	except KeyError, _:
             getLog.warn("Unknown MBOX user %r", info.user)
 
-        msg = _escapeMessageForEmail(message, info.tag)
+        msg = _escapeMessageForEmail(message, tag)
 
         fields = { 'user': address,
                    'return': self.returnAddress,
@@ -478,15 +478,16 @@
         self.tmpQueue = mixminion.Queue.queue(queueDir+"_tmp", 1, 1)
         self.tmpQueue.removeAll()
         return _MixmasterSMTPModuleDeliveryQueue(self, queueDir)
-    def processMessage(self, message, exitType, exitInfo):
+    def processMessage(self, message, tag, exitType, smtpAddress):
         assert exitType == SMTP_TYPE
-        info = mixminion.Packet.parseSMTPInfo(exitInfo)
+	# parseSMTPInfo will raise a parse error if the mailbox is invalid.
+        info = mixminion.Packet.parseSMTPInfo(smtpAddress)
 
-        msg = _escapeMessageForEmail(message, info.tag)
+        msg = _escapeMessageForEmail(message, tag)
         handle = self.tmpQueue.queueMessage(msg)
-        
+
         cmd = self.command
-        opts = self.options + (info.email,
+        opts = self.options + ("-t", info.email,
                                self.tmpQueue.getMessagePath(handle))
         code = os.spawnl(os.P_WAIT, cmd, cmd, *opts)
         getLog().debug("Queued Mixmaster message: exit code %s", code)
@@ -556,6 +557,8 @@
     """XXXX DOCDOC
 	  -> ("TXT"|"BIN"|"ENC", message, tag|None) or None
     """
+    if not tag:
+	raise ContentError("Missing tag from message")
     try:
 	message = mixminion.BuildMessage.decodePayload(payload, tag)
     except MixError, _:

Index: PacketHandler.py
===================================================================
RCS file: /home/minion/cvsroot/src/minion/lib/mixminion/PacketHandler.py,v
retrieving revision 1.8
retrieving revision 1.9
diff -u -d -r1.8 -r1.9
--- PacketHandler.py	10 Sep 2002 14:45:30 -0000	1.8
+++ PacketHandler.py	22 Nov 2002 21:12:05 -0000	1.9
@@ -58,8 +58,8 @@
            Returns one of:
                     None [if the mesesage should be dropped.]
                     ("EXIT",
-                       (routing_type, routing_info, application_key,
-                        payload)) [if this is the exit node]
+                       (exit_type, exit_info, application_key,
+                        tag, payload)) [if this is the exit node]
                     ("QUEUE", (ipv4info, message_out))
                         [if this is a forwarding node]
 
@@ -145,8 +145,9 @@
         # further.
         if rt >= Modules.MIN_EXIT_TYPE:
             return ("EXIT",
-                    (rt, subh.routinginfo,
+                    (rt, subh.getExitAddress(),
                      keys.get(Crypto.APPLICATION_KEY_MODE),
+		     subh.getTag(),
                      payload))
 
         # If we're not an exit node, make sure that what we recognize our

Index: ServerMain.py
===================================================================
RCS file: /home/minion/cvsroot/src/minion/lib/mixminion/ServerMain.py,v
retrieving revision 1.13
retrieving revision 1.14
diff -u -d -r1.13 -r1.14
--- ServerMain.py	22 Nov 2002 00:26:36 -0000	1.13
+++ ServerMain.py	22 Nov 2002 21:12:05 -0000	1.14
@@ -360,8 +360,8 @@
 	for h in handles:
 	    tp, info = self.queue.getObject(h)
 	    if tp == 'EXIT':
-		rt, ri, app_key, payload = info
-		self.moduleManager.queueMessage(payload, rt, ri)
+		rt, ri, app_key, tag, payload = info
+		self.moduleManager.queueMessage(payload, tag, rt, ri)
 	    else:
 		assert tp == 'QUEUE'
 		ipv4, msg = info
@@ -622,6 +622,7 @@
         sys.exit(1)
 
     config = readConfigFile(configFile)
+    mixminion.Common.configureShredCommand(config)
     getLog().setMinSeverity("INFO")
     keyring = ServerKeyring(config)
     keyring.checkKeys()

Index: benchmark.py
===================================================================
RCS file: /home/minion/cvsroot/src/minion/lib/mixminion/benchmark.py,v
retrieving revision 1.11
retrieving revision 1.12
diff -u -d -r1.11 -r1.12
--- benchmark.py	22 Nov 2002 00:21:20 -0000	1.11
+++ benchmark.py	22 Nov 2002 21:12:05 -0000	1.12
@@ -363,7 +363,7 @@
         tm = timeit_( \
               lambda np1=np1,np2=np2,it=it,serverinfo=serverinfo,
                       payload=payload: buildForwardMessage(payload,
-                                               1,
+                                               500,
                                                "Hello",
                                                serverinfo[:np1],
                                                serverinfo[:np2]), it)

Index: test.py
===================================================================
RCS file: /home/minion/cvsroot/src/minion/lib/mixminion/test.py,v
retrieving revision 1.37
retrieving revision 1.38
diff -u -d -r1.37 -r1.38
--- test.py	22 Nov 2002 00:21:20 -0000	1.37
+++ test.py	22 Nov 2002 21:12:05 -0000	1.38
@@ -605,12 +605,12 @@
 
         s = Subheader(3,9,"abcdeabcdeabcdef",
                       "ABCDEFGHIJABCDEFGHIJ",
-                      62, ts_eliot, len(ts_eliot))
+                      300, ts_eliot, len(ts_eliot))
 
         self.assertEquals(len(ts_eliot), 186)
 
         # test extended subeaders
-        expected = "\003\011abcdeabcdeabcdefABCDEFGHIJABCDEFGHIJ\000\272\000\076Who is the third who walks always beside you"
+        expected = "\003\011abcdeabcdeabcdefABCDEFGHIJABCDEFGHIJ\000\272\001\054Who is the third who walks always beside you"
         self.assertEquals(len(expected), mixminion.Packet.MAX_SUBHEADER_LEN)
         self.assertEquals(s.pack(), expected)
 
@@ -627,16 +627,18 @@
         self.assertEquals(s.minor, 9)
         self.assertEquals(s.secret, "abcde"*3+"f")
         self.assertEquals(s.digest, "ABCDEFGHIJ"*2)
-        self.assertEquals(s.routingtype, 62)
+        self.assertEquals(s.routingtype, 300)
         self.assertEquals(s.routinglen, 186)
         self.failUnless(s.isExtended())
         self.assertEquals(s.getNExtraBlocks(), 2)
 
         s.appendExtraBlocks("".join(extra))
         self.assertEquals(s.routinginfo, ts_eliot)
+	self.assertEquals(s.getExitAddress(), ts_eliot[20:])
+	self.assertEquals(s.getTag(), ts_eliot[:20])
         self.assertEquals(s.pack(), expected)
         self.assertEquals(s.getExtraBlocks(), extra)
-
+	
         # Underlong subheaders must fail
         self.failUnlessRaises(ParseError,
                               parseSubheader, "a"*(41))
@@ -681,21 +683,15 @@
         self.failUnlessRaises(ParseError, parseIPV4Info, ri[:-1])
         self.failUnlessRaises(ParseError, parseIPV4Info, ri+"x")
 
-
-    def test_smtpinfomboxinfo(self):
-	tag = "\x01\x02"+"Z"*18
-        for _class, _parse, _key in ((SMTPInfo, parseSMTPInfo, 'email'),
-                                     (MBOXInfo, parseMBOXInfo, 'user')):
-            ri = tag+"no-such-user@wangafu.net"
-            inf = _parse(ri)
-            self.assertEquals(getattr(inf,_key), "no-such-user@wangafu.net")
-            self.assertEquals(inf.tag, tag)
-            self.assertEquals(inf.pack(), ri)
-            inf = _class("no-such-user@wangafu.net",tag)
-            self.assertEquals(inf.pack(), ri)
-            # Message too short to have a tag
-            ri = "no-such-user@wangaf"
-	    self.failUnlessRaises(ParseError, _parse, ri)
+    def test_smtpinfo(self):
+	for addr in "Foo@bar.com", "a@b", "a@b.c.d.e", "a!b.c@d", "z@z":
+	    self.assertEquals(parseSMTPInfo(addr).pack(), addr)
+	    
+	for addr in ("(foo)@bar.com", "z.d" "z@", "@z", "@foo.com", "aaa",
+		     "foo.bar@", "foo\177@bar.com", "foo@bar\177.com",
+		     "foo@bar;cat /etc/shadow;echo ","foo bar@baz.com",
+		     "a@b@c"):
+	    self.failUnlessRaises(ParseError, parseSMTPInfo, addr)
 
     def test_replyblock(self):
 	key = "\x99"*16
@@ -853,6 +849,7 @@
 
 #----------------------------------------------------------------------
 import mixminion.BuildMessage as BuildMessage
+import mixminion.Modules
 from mixminion.Modules import *
 
 class FakePRNG:
@@ -1074,13 +1071,14 @@
         longStr2 = longStr * 2
 
         def getLongRoutingInfo(longStr2=longStr2,tag=tag):
-            return MBOXInfo(longStr2,tag)
+            return MBOXInfo(tag+longStr2)
 
         server4 = FakeServerInfo("127.0.0.1", 1, self.pk1, "X"*20)
         server4.getRoutingInfo = getLongRoutingInfo
 
         secrets.append("1"*16)
-        head = bhead([self.server2, server4], secrets, 99, longStr,
+        head = bhead([self.server2, server4], secrets, 99, 
+		     longStr,
                      AESCounterPRNG())
         pks = (self.pk2,self.pk1)
         rtypes = (FWD_TYPE,99)
@@ -1577,11 +1575,12 @@
             else:
                 self.assertEquals(res[0], "EXIT")
                 self.assertEquals(res[1][0], rt)
-		self.assertEquals(res[1][1][20:], ri)
+		self.assertEquals(res[1][1], ri)
                 if appkey:
                     self.assertEquals(appkey, res[1][2])
 		
-		p = res[1][3]
+		#tag = res[1][3]
+		p = res[1][4]
 		p = BuildMessage.decodeForwardPayload(p)
                 self.assert_(p.startswith(payload))
                 break
@@ -1592,7 +1591,7 @@
         p = "Now is the time for all good men to come to the aid"
         m = bfm(p, SMTP_TYPE, "nobody@invalid",
                 [self.server1, self.server2], [self.server3])
-
+	
         self.do_test_chain(m,
                            [self.sp1,self.sp2,self.sp3],
                            [FWD_TYPE, FWD_TYPE, SMTP_TYPE],
@@ -1701,9 +1700,13 @@
         self.failUnlessRaises(ParseError, self.sp1.processMessage, m_x)
 
         # Bad internal type
-        m_x = bfm("Z", 50, "", [self.server1], [self.server2])
-        q, (a,m_x) = self.sp1.processMessage(m_x)
-        self.failUnlessRaises(ContentError, self.sp2.processMessage, m_x)
+	try:
+	    save = mixminion.Modules.SWAP_FWD_TYPE
+	    mixminion.Modules.SWAP_FWD_TYPE = 50
+	    m_x = bfm("Z", 500, "", [self.server1], [self.server2])
+	finally:
+	    mixminion.Modules.SWAP_FWD_TYPE = save
+        self.failUnlessRaises(ContentError, self.sp1.processMessage, m_x)
 
         # Subhead we can't parse
         m_x = pk_encrypt("foo", self.pk1)+m[128:]
@@ -2739,7 +2742,7 @@
 	    return None
     def getExitTypes(self):
 	return (1234,)
-    def processMessage(self, message, exitType, exitInfo):
+    def processMessage(self, message, tag, exitType, exitInfo):
 	self.processedMessages.append(message)
 	if exitInfo == 'fail?':
 	    return mixminion.Modules.DELIVER_FAIL_RETRY
@@ -2799,18 +2802,18 @@
 	manager.configure(conf)
 	self.failUnless(exampleMod is manager.typeToModule[1234])
 
-	manager.queueMessage("Hello 1", 1234, "fail!")
-	manager.queueMessage("Hello 2", 1234, "fail?")
-	manager.queueMessage("Hello 3", 1234, "good")
-	manager.queueMessage("Drop very much",
-			     mixminion.Modules.DROP_TYPE,  "")
+	t = "ZZZZ"*5
+	manager.queueMessage("Hello 1", t, 1234, "fail!")
+	manager.queueMessage("Hello 2", t, 1234, "fail?")
+	manager.queueMessage("Hello 3", t, 1234, "good")
+	manager.queueMessage("Drop very much", None,
+			     mixminion.Modules.DROP_TYPE, t)
 	queue = manager.queues['TestModule']
 	self.failUnless(isinstance(queue,
 			   mixminion.Modules.SimpleModuleDeliveryQueue))
 	self.assertEquals(3, queue.count())
 	self.assertEquals(exampleMod.processedMessages, [])
 	try:
-	    severity = getLog().getMinSeverity()
 	    suspendLog()
 	    manager.sendReadyMessages()
 	finally:
@@ -3087,6 +3090,9 @@
 
 class ClientMainTests(unittest.TestCase):
     def testClientKeystore(self):
+	if 1:
+	    return
+	#XXXX
 	eq = self.assertEquals
 	raises = self.failUnlessRaises
 

Index: testSupport.py
===================================================================
RCS file: /home/minion/cvsroot/src/minion/lib/mixminion/testSupport.py,v
retrieving revision 1.6
retrieving revision 1.7
diff -u -d -r1.6 -r1.7
--- testSupport.py	21 Nov 2002 19:43:29 -0000	1.6
+++ testSupport.py	22 Nov 2002 21:12:05 -0000	1.7
@@ -69,13 +69,8 @@
 	else:
 	    return ImmediateDeliveryQueue(self)
 	
-    def processMessage(self, message, exitType, exitInfo):
+    def processMessage(self, message, tag, exitType, exitInfo):
 	assert exitType == 0xFFFE
-	if len(exitInfo) > 20:
-	    tag = exitInfo[:20]
-	    exitInfo = exitInfo[20:]
-	else:
-	    tag = "\000"*20
 
 	if exitInfo == 'fail':
 	    return DELIVER_FAIL_RETRY
@@ -153,7 +148,8 @@
 		    print "Directory %s has fishy permissions %o" %(parent,m)
 		    sys.exit(1)
 		if st[stat.ST_UID] not in (0, os.getuid()):
-		    print "Directory %s has bad owner %s" % st[stat.ST_UID]
+		    print "Directory %s has bad owner %s" % (parent, 
+							     st[stat.ST_UID])
 		    sys.exit(1)
 		    
 	_MM_TESTING_TEMPDIR = temp
Index: Packet.py
===================================================================
RCS file: /home/minion/cvsroot/src/minion/lib/mixminion/Packet.py,v
retrieving revision 1.14
retrieving revision 1.15
diff -u -d -r1.14 -r1.15
--- Packet.py	21 Nov 2002 16:55:49 -0000	1.14
+++ Packet.py	22 Nov 2002 21:12:05 -0000	1.15
@@ -16,9 +16,11 @@
 	    'SINGLETON_PAYLOAD_OVERHEAD', 'OAEP_OVERHEAD',
 	    'FRAGMENT_PAYLOAD_OVERHEAD', 'ENC_FWD_OVERHEAD']
 
+import re
 import struct
 from socket import inet_ntoa, inet_aton
 from mixminion.Common import MixError, floorDiv
+import mixminion.Modules
 
 # Major and minor number for the understood packet format.
 MAJOR_NO, MINOR_NO = 0,1
@@ -138,6 +140,8 @@
     ri = s[MIN_SUBHEADER_LEN:]
     if rlen < len(ri):
         ri = ri[:rlen]
+    if rt >= mixminion.Modules.MIN_EXIT_TYPE and rlen < 20:
+	raise ParseError("Subheader missing tag")
     return Subheader(major,minor,secret,digest,rt,ri,rlen)
 
 def getTotalBlocksForRoutingInfoLen(bytes):
@@ -180,6 +184,18 @@
                 "routingtype=%(routingtype)r, routinginfo=%(routinginfo)r, "+
                 "routinglen=%(routinglen)r)")% self.__dict__
 
+    def getExitAddress(self):
+	# XXXX SPEC This is not explicit in the spec.
+	assert self.routingtype >= mixminion.Modules.MIN_EXIT_TYPE
+	assert len(self.routinginfo) >= TAG_LEN
+	return self.routinginfo[TAG_LEN:]
+    
+    def getTag(self):
+	# XXXX SPEC This is not explicit in the spec.
+	assert self.routingtype >= mixminion.Modules.MIN_EXIT_TYPE
+	assert len(self.routinginfo) >= TAG_LEN
+	return self.routinginfo[:TAG_LEN]
+
     def setRoutingInfo(self, info):
         """Change the routinginfo, and the routinglength to correspond."""
         self.routinginfo = info
@@ -437,41 +453,41 @@
 	return (type(self) == type(other) and self.ip == other.ip and
 		self.port == other.port and self.keyinfo == other.keyinfo)
 
+# XXXX Support subdomains and quotesd strings
+_ATOM_PAT = r'[^\x00-\x20()\[\]()<>@,;:\\".\x7f-\xff]+'
+_LOCAL_PART_PAT = r"(?:%s)(?:\.(?:%s))*" % (_ATOM_PAT, _ATOM_PAT)
+_RFC822_PAT = r"\A%s@%s\Z" % (_LOCAL_PART_PAT, _LOCAL_PART_PAT)
+RFC822_RE = re.compile(_RFC822_PAT)
+
 def parseSMTPInfo(s):
-    """Convert the encoding of an SMTP routinginfo into an SMTPInfo object."""
-    if len(s) <= TAG_LEN:
-	raise ParseError("SMTP routing info is too short")
-    return SMTPInfo(s[TAG_LEN:], s[:TAG_LEN])
+    """Convert the encoding of an SMTP exitinfo into an SMTPInfo object."""
+    m = RFC822_RE.match(s)
+    if not m:
+	raise ParseError("Invalid rfc822 mailbox %r" % s)
+    return SMTPInfo(s)
 
 class SMTPInfo:
     """Represents the routinginfo for an SMTP hop.
 
-       Fields: email (an email address), tag (20 bytes)."""
-    def __init__(self, email, tag):
-	assert len(tag) == TAG_LEN
+       Fields: email (an email address)"""
+    def __init__(self, email):
         self.email = email
-        self.tag = tag
 
     def pack(self):
         """Returns the wire representation of this SMTPInfo"""
-	return self.tag + self.email
+	return self.email
 
 def parseMBOXInfo(s):
-    """Convert the encoding of an MBOX routinginfo into an MBOXInfo
-       object."""
-    if len(s) < TAG_LEN:
-	raise ParseError("MBOX routing info is too short")
-    return MBOXInfo(s[TAG_LEN:], s[:TAG_LEN])
+    """Convert the encoding of an MBOX exitinfo into an MBOXInfo address"""
+    return MBOXInfo(s)
 
 class MBOXInfo:
-    """Represents the routinginfo for an MBOX hop.
+    """Represents the exitinfo for an MBOX hop.
 
-       Fields: user (a user identifier), tag (20 bytes)."""
-    def __init__(self, user, tag):
-	assert len(tag) == TAG_LEN
+       Fields: user (a user identifier)"""
+    def __init__(self, user):
         self.user = user
-        self.tag = tag
 
     def pack(self):
         """Return the external representation of this routing info."""
-	return self.tag + self.user
+	return self.user
@@ -437,41 +453,41 @@
 
 class SMTPInfo:
-    """Represents the routinginfo for an SMTP hop.
+    """Represents the exit address for an SMTP hop.
 
